import glob

from pathlib import Path
from os import walk


def read_template(file_name: str) -> str:
    """Reads a template file and returns its content

    Args:
        file_name: str, Required
        Template file name

    Returns:
        Template file contents
    """
    return Path(file_name).read_text()


def prepare_output_path_dir(dir_name: str):
    """Creates missing directories for output files

    Args:
        dir_name: str, Required
        Name of output directory
    """
    if not Path(dir_name).is_dir():
        print(f'Creating dir named: {dir_name}')
        Path(dir_name).mkdir()


def guarantee_trailing_slash(directory_name: str) -> str:
    """Adds a trailling / when missing
    """
    if not directory_name.endswith('/'):
        return directory_name + '/'
    return directory_name


def prepare_files_and_subfolders(templates_dir: str = 'templates/'):
    # list files in templates
    walked = list(walk(templates_dir))
    files = walked[0][2]  # files inside templates/ and outside of subfolders
    subfolders = walked[0][1]  # always a list of subfolders
    templates_dir_quantity = walked.__len__()

    # if templates has subfolder structure this list all templates in subfold
    if templates_dir_quantity > 1:
        files = glob.glob(templates_dir + '*/*')

    return files, subfolders, templates_dir_quantity


def create_subfolder_structure(subfolders, output_path):
    """Creates subfolders inside output path

    Args:
        subfolders: list(str), Required
        A list of string cotaining templates/ subfolders names

        output_path: str, Required
        Output directory to write rendered templates
    """
    for subf in subfolders:
        prepare_output_path_dir(f'{output_path}{subf}')



from configparser import ConfigParser


def variables_to_substitute(variable_list):
    parser = ConfigParser()
    parser.read(variable_list)
    try:
        variables = dict(parser['variables'])
    except KeyError:
        print(f'Oh no! No "variables" section found in {variable_list}')
        return
    return variables



from pathlib import Path

import click
from jinja2 import Template

from .configuration import variables_to_substitute
from .files_helper import guarantee_trailing_slash, create_subfolder_structure
from .files_helper import prepare_files_and_subfolders
from .files_helper import prepare_output_path_dir, read_template


def run(variables: dict, output_path: str):
    """Runs the logic for generating rendered templates outcome

    Args:
        dir_name: str, Required
        Name of output directory
    """
    # avoids problem with concats without trailling slash
    output_path = guarantee_trailing_slash(output_path)

    # makes sure output dir exists to avoid FileNotFound error
    prepare_output_path_dir(output_path)

    file_list, subfolders, dir_size = prepare_files_and_subfolders()

    if subfolders:
        create_subfolder_structure(subfolders, output_path)

    for f in file_list:
        # makes sure I have correct file names for subfolder structure
        if dir_size == 1:
            file_name = f
            f = f'templates/{f}'
        else:
            file_name = f.split('templates/')[1]
        template = read_template(f)

        # create templates
        file_template = Template(template)
        file_name_template = Template(file_name)

        # rendering templates
        output_file_name = file_name_template.render(variables)
        output_file = file_template.render(variables)

        # writing outputs
        with open(f'{output_path}/{output_file_name}', 'w') as f:
            click.echo(f'Writting file: {output_file_name}')
            f.write(output_file)


@click.command()
@click.option('-v', '--variable-list', default='variables.yaml',
              help='Name of the yaml files containing variables')
@click.option('-o', '--output-path', default='./',
              help='Output path, by default writes to root')
def AutoGenFiles(variable_list, output_path):
    print(f'Output path: {output_path}')
    # read variables to substitute
    if not Path(variable_list).is_file():
        print(f'Oh no! No file named {variable_list} found!')
        return

    variables = variables_to_substitute(variable_list)
    if not variables:
        return
    click.echo(f'Variables found: {list(variables.keys())}')

    run(variables, output_path)
    click.echo('Output fully written')



from setuptools import setup

REPO_URL = 'http://github.com/jtemporal/autogenfiles'

with open('README.md') as fobj:
    long_description = fobj.read()

setup(
    author='Jessica Temporal',
    author_email='hello@jtemporal.com',
    description='Auto generate files based on a template',
    zip_safe=False,
    install_requires=[
        'click==8.0.1',
        'Jinja2==3.0.1'
    ],
    keywords='auto generate files, jinja, templates, cli',
    license='MIT',
    long_description=long_description,
    name='autogenfiles',
    packages=[
        'autogenfiles'
    ],
    entry_points={
        'console_scripts': [
            'autogenfiles = autogenfiles.autogenfiles:AutoGenFiles',
        ],
    },
    url=REPO_URL,
    python_requires='>=3.8.2',
    version='0.0.4',
)
